@@ -52,6 +52,12 @@ sub class_id {
     return join '-', map lc, @pkg;
 }
 
+# subclasses may overload to avoid cache sharing
+sub plugin_id {
+    my $self = shift;
+    $self->class_id;
+}
+
 sub assets_dir {
     my $self = shift;
 
@@ -62,4 +68,9 @@ sub assets_dir {
 
 }
 
+sub log {
+    my $self = shift;
+    Plagger->context->log(@_, caller => ref($self));
+}
+
 1;
